@@ -70,7 +70,12 @@ public:
     sigc::signal<void>& detached_from_target_signal () const;
 
     sigc::signal<void, const map<int, IDebugger::Breakpoint>&, const UString&>&
-                                            breakpoints_set_signal () const;
+                                            breakpoints_list_signal () const;
+
+    sigc::signal<void,
+                const std::pair<int, const IDebugger::Breakpoint&>&,
+                const UString& /*cookie*/>&
+        breakpoint_set_signal () const;
 
     sigc::signal<void, const vector<OverloadsChoiceEntry>&, const UString&>&
                                     got_overloads_choice_signal () const;
@@ -143,7 +148,7 @@ public:
     sigc::signal<void, const VariableSafePtr, const UString&>&
                                     variable_type_set_signal () const;
 
-    sigc::signal<void, const VariableSafePtr, const UString&>
+    sigc::signal<void, const VariableSafePtr, const UString&>&
                                       variable_dereferenced_signal () const;
 
     sigc::signal<void, int, const UString&>& got_target_info_signal () const ;
@@ -246,13 +251,15 @@ public:
 
     bool load_program (const UString &a_prog,
                        const vector<UString> &a_args,
-                       const UString &a_working_dir);
+                       const UString &a_working_dir,
+		       bool a_force);
 
     bool load_program (const UString &a_prog,
                        const vector<UString> &a_argv,
                        const UString &working_dir,
                        const vector<UString> &a_source_search_dirs,
-                       const UString &a_tty_path);
+                       const UString &a_tty_path,
+		       bool a_force);
 
     void load_core_file (const UString &a_prog_file,
                          const UString &a_core_path);
@@ -326,11 +333,20 @@ public:
                                gint a_line_num,
                                const UString &a_cookie) ;
 
+    void jump_to_position (const Loc &a_loc,
+			   const DefaultSlot &a_slot);
+
+    void set_breakpoint (const Loc &a_loc,
+			 const UString &a_condition,
+			 gint a_ignore_count,
+			 const BreakpointSlot &a_slot,
+			 const UString &a_cookie);
+
     void set_breakpoint (const UString &a_path,
                          gint a_line_num,
                          const UString &a_condition,
                          gint a_ignore_count,
-                         const UString &a_cookie) ;
+                         const UString &a_cookie);
 
     void set_breakpoint (const UString &a_func_name,
                          const UString &a_condition,
@@ -342,6 +358,10 @@ public:
                          gint a_ignore_count,
                          const UString &a_cookie);
 
+    void enable_breakpoint (gint a_break_num,
+			    const BreakpointsSlot &a_slot,
+			    const UString &a_cookie = "");
+
     void enable_breakpoint (gint a_break_num,
                             const UString &a_cookie="");
 
@@ -406,10 +426,20 @@ public:
                       int a_high_frame,
                       const UString &a_cookie);
 
+    void list_frames (int a_low_frame,
+		      int a_high_frame,
+		      const FrameVectorSlot &a_slot,
+		      const UString &a_cookie);
+
     void list_frames_arguments (int a_low_frame,
                                 int a_high_frame,
                                 const UString &a_cookie);
 
+    void list_frames_arguments (int a_low_frame,
+				int a_high_frame,
+				const FrameArgsSlot &a_slot,
+				const UString &a_cookie);
+
     void list_local_variables (const UString &a_cookie);
 
     void list_global_variables ( const UString &a_cookie );
