@@ -35,10 +35,11 @@
 #include "common/nmv-ustring.h"
 #include "common/nmv-dynamic-module.h"
 #include "common/nmv-safe-ptr-utils.h"
+#include "common/nmv-address.h"
+#include "common/nmv-asm-instr.h"
+#include "common/nmv-loc.h"
 #include "nmv-i-conf-mgr.h"
 
-NEMIVER_BEGIN_NAMESPACE (nemiver)
-
 using nemiver::common::SafePtr;
 using nemiver::common::DynamicModule;
 using nemiver::common::DynamicModuleSafePtr;
@@ -47,11 +48,19 @@ using nemiver::common::ObjectRef;
 using nemiver::common::ObjectUnref;
 using nemiver::common::UString;
 using nemiver::common::Object;
+using nemiver::common::Address;
+using nemiver::common::AsmInstr;
+using nemiver::common::MixedAsmInstr;
+using nemiver::common::Asm;
+using nemiver::common::DisassembleInfo;
+using nemiver::common::Loc;
 using std::vector;
 using std::string;
 using std::map;
 using std::list;
 
+NEMIVER_BEGIN_NAMESPACE (nemiver)
+
 class ILangTrait;
 class IDebugger;
 typedef SafePtr<IDebugger, ObjectRef, ObjectUnref> IDebuggerSafePtr;
@@ -77,31 +86,34 @@ public:
     typedef unsigned int register_id_t;
 
     /// \brief a breakpoint descriptor
-    class BreakPoint {
+    class Breakpoint {
     public:
 
         enum Type {
             UNDEFINED_TYPE = 0,
             STANDARD_BREAKPOINT_TYPE,
-            WATCHPOINT_TYPE
+            WATCHPOINT_TYPE,
+            COUNTPOINT_TYPE
         };
 
     private:
         int m_number;
         bool m_enabled;
-        UString m_address;
-        UString m_function;
-        UString m_expression;
+        Address m_address;
+        string m_function;
+        string m_expression;
         UString m_file_name;
         UString m_file_full_name;
-        UString m_condition;
+        string m_condition;
         Type m_type;
         int m_line;
         int m_nb_times_hit;
         int m_ignore_count;
+        bool m_is_read_watchpoint;
+        bool m_is_write_watchpoint;
 
     public:
-        BreakPoint () {clear ();}
+        Breakpoint () {clear ();}
 
         /// \name accessors
 
@@ -112,14 +124,15 @@ public:
         bool enabled () const {return m_enabled;}
         void enabled (bool a_in) {m_enabled = a_in;}
 
-        const UString& address () const {return m_address;}
-        void address (const UString &a_in) {m_address = a_in;}
+        const Address& address () const {return m_address;}
+        Address& address () {return m_address;}
+        void address (const string &a_in) {m_address = a_in;}
 
-        const UString& function () const {return m_function;}
-        void function (const UString &a_in) {m_function = a_in;}
+        const string& function () const {return m_function;}
+        void function (const string &a_in) {m_function = a_in;}
 
-        const UString& expression () const {return m_expression;}
-        void expression (const UString &a_expr) {m_expression = a_expr;}
+        const string& expression () const {return m_expression;}
+        void expression (const string &a_expr) {m_expression = a_expr;}
 
         const UString& file_name () const {return m_file_name;}
         void file_name (const UString &a_in) {m_file_name = a_in;}
@@ -130,8 +143,8 @@ public:
         int line () const {return m_line;}
         void line (int a_in) {m_line = a_in;}
 
-        const UString& condition () const {return m_condition;}
-        void condition (const UString &a_cond) {m_condition = a_cond;}
+        const string& condition () const {return m_condition;}
+        void condition (const string &a_cond) {m_condition = a_cond;}
 
         bool has_condition () const {return !m_condition.empty ();}
 
@@ -141,6 +154,12 @@ public:
         int ignore_count () const {return m_ignore_count;}
         void ignore_count (int a) {m_ignore_count = a;}
 
+        bool is_read_watchpoint () const {return m_is_read_watchpoint;}
+        void is_read_watchpoint (bool f) {m_is_read_watchpoint = f;}
+
+        bool is_write_watchpoint () const {return m_is_write_watchpoint;}
+        void is_write_watchpoint (bool f) {m_is_write_watchpoint = f;}
+
         bool is_pending ()
         {
             if (m_address == "<PENDING>") {
@@ -157,6 +176,7 @@ public:
         /// \brief clear this instance of breakpoint
         void clear ()
         {
+            m_type = STANDARD_BREAKPOINT_TYPE;
             m_number = 0;
             m_enabled = false;
             m_address.clear ();
@@ -167,8 +187,10 @@ public:
             m_condition.clear ();
             m_nb_times_hit = 0;
             m_ignore_count = 0;
+            m_is_read_watchpoint = false;
+            m_is_write_watchpoint = false;
         }
-    };//end class BreakPoint
+    };//end class Breakpoint
 
     /// \brief an entry of a choice list
     /// to choose between a list of overloaded
@@ -236,11 +258,15 @@ public:
         void line_number (int a_in) {m_line_number = a_in;}
     };//end class OverloadsChoiceEntry
 
+    class Variable;
+    typedef SafePtr<Variable, ObjectRef, ObjectUnref> VariableSafePtr;
+    typedef list<VariableSafePtr> VariableList;
+
     /// \brief a function frame as seen by the debugger.
     class Frame {
-        UString m_address;
-        UString m_function_name;
-        map<UString, UString> m_args;
+        Address m_address;
+        string m_function_name;
+        map<string, string> m_args;
         int m_level;
         //present if the target has debugging info
         UString m_file_name;
@@ -248,25 +274,40 @@ public:
         UString m_file_full_name;
         int m_line;
         //present if the target doesn't have debugging info
-        UString m_library;
+        string m_library;
     public:
 
         Frame () :
-            m_level (0),
+        m_level (0),
             m_line (0)
-        {}
+            {
+            }
 
+        /// \operators
+        /// @{
+
+        bool operator== (const Frame &a)
+        {
+            return (address () == a.address ());
+        }
+
+        /// @}
         /// \name accessors
 
         /// @{
-        const UString& address () const {return m_address;}
-        void address (const UString &a_in) {m_address = a_in;}
+        const Address& address () const {return m_address;}
+        Address& address () {return m_address;}
+        void address (const Address &a_in) {m_address = a_in;}
+        bool has_empty_address () const
+        {
+            return m_address.to_string ().empty ();
+        }
 
-        const UString& function_name () const {return m_function_name;}
-        void function_name (const UString &a_in) {m_function_name = a_in;}
+        const string& function_name () const {return m_function_name;}
+        void function_name (const string &a_in) {m_function_name = a_in;}
 
-        const map<UString, UString>& args () const {return m_args;}
-        map<UString, UString>& args () {return m_args;}
+        const map<string, string>& args () const {return m_args;}
+        map<string, string>& args () {return m_args;}
 
         int level () const {return m_level;}
         void level (int a_level) {m_level = a_level;}
@@ -280,8 +321,8 @@ public:
         int line () const {return m_line;}
         void line (int a_in) {m_line = a_in;}
 
-        const UString& library () const {return m_library;}
-        void library (const UString &a_library) {m_library = a_library;}
+        const string& library () const {return m_library;}
+        void library (const string &a_library) {m_library = a_library;}
 
         /// @}
 
@@ -299,10 +340,11 @@ public:
             m_args.clear ();
         }
     };//end class Frame
+    typedef sigc::slot<void, const vector<IDebugger::Frame>&>
+        FrameVectorSlot;
+    typedef sigc::slot<void, const map<int, IDebugger::VariableList>& >
+        FrameArgsSlot;
 
-    class Variable;
-    typedef SafePtr<Variable, ObjectRef, ObjectUnref> VariableSafePtr;
-    typedef list<VariableSafePtr> VariableList;
     class Variable : public Object {
         // non copyable.
         Variable (const Variable &);
@@ -331,43 +373,52 @@ public:
         // If empty, it can be set by calling
         // IDebugger::query_variable_path_expr()
         UString m_path_expression;
+        bool m_in_scope;
 
     public:
         Variable (const UString &a_internal_name,
                   const UString &a_name,
                   const UString &a_value,
-                  const UString &a_type) :
-            m_internal_name (a_internal_name),
+                  const UString &a_type,
+                  bool a_in_scope = true)
+            : m_internal_name (a_internal_name),
             m_name (a_name),
             m_value (a_value),
             m_type (a_type),
             m_parent (0),
-            m_num_expected_children (0)
+            m_num_expected_children (0),
+            m_in_scope (a_in_scope)
 
         {
         }
 
         Variable (const UString &a_name,
                   const UString &a_value,
-                  const UString &a_type) :
-            m_name (a_name),
+                  const UString &a_type,
+                  bool a_in_scope = true)
+            : m_name (a_name),
             m_value (a_value),
             m_type (a_type),
             m_parent (0),
-            m_num_expected_children (0)
+            m_num_expected_children (0),
+            m_in_scope (a_in_scope)
 
         {
         }
 
-        Variable (const UString &a_name) :
-            m_name (a_name),
+        Variable (const UString &a_name)
+            : m_name (a_name),
             m_parent (0),
-            m_num_expected_children (0)
+            m_num_expected_children (0),
+            m_in_scope (true)
+                
         {}
 
-        Variable () :
-            m_parent (0),
-            m_num_expected_children (0)
+        Variable ()
+            : m_parent (0),
+            m_num_expected_children (0),
+            m_in_scope (true)
+                
         {}
 
         const VariableList& members () const {return m_members;}
@@ -711,6 +762,10 @@ public:
         {
             m_path_expression = a_expr;
         }
+
+        bool in_scope () const {return m_in_scope;}
+        void in_scope (bool a) {m_in_scope = a;}
+
     };//end class Variable
 
     enum State {
@@ -756,6 +811,28 @@ public:
         SIGNAL_RECEIVED
     };//end enum StopReason
 
+    /// Return true if a StopReason represents a reason for exiting.
+    static bool is_exited (enum StopReason a_reason)
+    {
+        if (a_reason == EXITED_SIGNALLED
+            || a_reason == EXITED
+            || a_reason == EXITED_NORMALLY)
+            return true;
+        return false;
+    }
+
+    typedef sigc::slot<void> DefaultSlot;
+
+    typedef sigc::slot<void,
+                       const std::pair<int, const IDebugger::Breakpoint&>&>
+        BreakpointSlot;
+
+    typedef sigc::slot<void,
+                       const std::map<int, IDebugger::Breakpoint>&>
+        BreakpointsSlot;
+
+    typedef sigc::slot<void, Loc&> LocSlot;
+    
     virtual ~IDebugger () {}
 
     /// \name events you can connect to.
@@ -784,7 +861,7 @@ public:
     virtual sigc::signal<void>& detached_from_target_signal () const=0;
 
     virtual sigc::signal<void,
-                        const IDebugger::BreakPoint&,
+                        const IDebugger::Breakpoint&,
                         int /*breakpoint command*/,
                         const UString & /*cookie*/>&
                                      breakpoint_deleted_signal () const=0;
@@ -795,9 +872,14 @@ public:
     /// IDebugger does not cache the list of breakpoints. This must
     /// be fixed at some point.
     virtual sigc::signal<void,
-                         const map<int, IDebugger::BreakPoint>&,
+                         const map<int, IDebugger::Breakpoint>&,
                          const UString& /*cookie*/>&
-                                         breakpoints_set_signal () const=0;
+                                         breakpoints_list_signal () const=0;
+
+    virtual sigc::signal<void,
+                        const std::pair<int, const IDebugger::Breakpoint&>&,
+                        const UString& /*cookie*/>&
+                        breakpoint_set_signal () const = 0;
 
     virtual sigc::signal<void,
                          const vector<OverloadsChoiceEntry>&,
@@ -883,7 +965,7 @@ public:
     /// Emitted as a the result of the IDebugger::dereference_variable is call.
     virtual sigc::signal<void,
                          const VariableSafePtr/*the variable we derefed*/,
-                         const UString&/*cookie*/>
+                         const UString&/*cookie*/>&
                                       variable_dereferenced_signal () const=0;
 
     virtual sigc::signal<void, const vector<UString>&, const UString&>&
@@ -892,7 +974,7 @@ public:
     virtual sigc::signal<void,
                          int/*pid*/,
                          const UString&/*target path*/>&
-                                            got_target_info_signal () const=0;
+                                            got_target_info_signal () const = 0;
 
     virtual sigc::signal<void>& running_signal () const=0;
 
@@ -904,52 +986,70 @@ public:
     virtual sigc::signal<void, const UString&/*error message*/>&
                                                     error_signal () const = 0;
 
-    virtual sigc::signal<void, IDebugger::State>& state_changed_signal () const=0;
+    virtual sigc::signal<void, IDebugger::State>&
+                                        state_changed_signal () const = 0;
 
-    virtual sigc::signal<void, const std::map<register_id_t, UString>&, const UString& >&
-                                                 register_names_listed_signal () const=0;
+    virtual sigc::signal<void,
+                         const std::map<register_id_t, UString>&,
+                         const UString& >&
+                                 register_names_listed_signal () const = 0;
+
+    virtual sigc::signal<void,
+                         const std::map<register_id_t, UString>&,
+                         const UString& >&
+                                   register_values_listed_signal () const = 0;
 
-    virtual sigc::signal<void, const std::map<register_id_t, UString>&, const UString& >&
-                                                 register_values_listed_signal () const=0;
     virtual sigc::signal<void,
                          const UString&/*register name*/,
                          const UString&/*register value*/,
                          const UString&/*cookie*/>&
-                                                 register_value_changed_signal () const=0;
-    virtual sigc::signal<void, const std::list<register_id_t>&, const UString& >&
-                             changed_registers_listed_signal () const=0;
+                                  register_value_changed_signal () const = 0;
+
+    virtual sigc::signal<void,
+                         const std::list<register_id_t>&,
+                         const UString& >&
+                                   changed_registers_listed_signal () const = 0;
 
     virtual sigc::signal <void,
                           size_t,/*start address*/
                           const std::vector<uint8_t>&,/*values*/
                           const UString&>&/*cookie*/
-                                          read_memory_signal () const = 0;
+                                 read_memory_signal () const = 0;
     virtual sigc::signal <void,
                           size_t,/*start address*/
                           const std::vector<uint8_t>&,/*values*/
                           const UString& >&
-                                          set_memory_signal () const = 0;
+                                 set_memory_signal () const = 0;
+
+    // TODO: export informations about what file is being disassembled,
+    // what function, which line (if possible) etc.
+    // So that the code receiving the signal can adjust accordingly
+    virtual sigc::signal<void,
+                         const DisassembleInfo&,
+                         const std::list<Asm>&,
+                         const UString& /*cookie*/>&
+                             instructions_disassembled_signal () const = 0;
 
     virtual sigc::signal<void, const VariableSafePtr, const UString&>&
-                                        variable_created_signal () const = 0;
+                                 variable_created_signal () const = 0;
 
     virtual sigc::signal<void, const VariableSafePtr, const UString&>&
-                                        variable_deleted_signal () const = 0;
+                                 variable_deleted_signal () const = 0;
 
     virtual sigc::signal<void, const VariableSafePtr, const UString&>&
-                                        variable_unfolded_signal () const = 0;
+                                 variable_unfolded_signal () const = 0;
 
     virtual sigc::signal<void, const VariableSafePtr, const UString&>&
-                variable_expression_evaluated_signal () const = 0;
+                                 variable_expression_evaluated_signal () const = 0;
 
     virtual sigc::signal<void, const VariableList&, const UString&>&
-                changed_variables_signal () const  = 0;
+                                changed_variables_signal () const  = 0;
 
     virtual sigc::signal<void, VariableSafePtr, const UString&>&
                 assigned_variable_signal () const = 0;
     /// @}
 
-    virtual void do_init (IConfMgrSafePtr &a_conf_mgr) = 0;
+    virtual void do_init (IConfMgrSafePtr a_conf_mgr) = 0;
 
     virtual map<UString, UString>& properties () = 0;
 
@@ -964,16 +1064,18 @@ public:
 
     virtual void set_solib_prefix_path (const UString &a_name) = 0;
 
-    virtual void load_program (const UString &a_prog,
+    virtual bool load_program (const UString &a_prog,
                                const vector<UString> &a_args,
-                               const UString &a_working_dir) = 0;
+                               const UString &a_working_dir,
+                               bool a_force = false) = 0;
 
-    virtual void load_program
+    virtual bool load_program
                 (const UString &a_prog,
                  const vector<UString> &a_argv,
                  const UString &working_dir,
                  const vector<UString> &a_source_search_dirs,
-                 const UString &a_tty_path="") = 0;
+                 const UString &a_tty_path,
+                 bool a_force = false) = 0;
 
     virtual void load_core_file (const UString &a_prog_file,
                                  const UString &a_core_file) = 0;
@@ -982,13 +1084,17 @@ public:
                                    const UString &a_tty_path="") = 0;
 
     virtual bool attach_to_remote_target (const UString &a_host,
-                                          int a_port) = 0;
+                                          unsigned a_port) = 0;
 
     virtual bool attach_to_remote_target (const UString &a_serial_line) = 0;
 
     virtual void detach_from_target (const UString &a_cookie="") = 0;
 
-    virtual bool is_attached_to_target () const =0;
+    virtual void disconnect_from_remote_target (const UString &a_cookie = "") = 0;
+
+    virtual bool is_attached_to_target () const = 0;
+
+    virtual void set_tty_path (const UString &a_tty_path) = 0;
 
     virtual void add_env_variables (const map<UString, UString> &a_vars) = 0;
 
@@ -1014,24 +1120,48 @@ public:
 
     virtual void exit_engine () = 0;
 
-    virtual void step_in (const UString &a_cookie="") = 0;
-
     virtual void step_over (const UString &a_cookie="") = 0;
 
+    virtual void step_in (const UString &a_cookie="") = 0;
+
     virtual void step_out (const UString &a_cookie="") = 0;
 
-    virtual void step_instruction (const UString &a_cookie="") = 0;
+    virtual void step_over_asm (const UString &a_cookie="") = 0;
+
+    virtual void step_in_asm (const UString &a_cookie="") = 0;
 
     virtual void continue_to_position (const UString &a_path,
                                        gint a_line_num,
                                        const UString &a_cookie="") = 0;
+
+    virtual void jump_to_position (const Loc &a_loc,
+                                   const DefaultSlot &a_slot) = 0;
+
+    virtual void set_breakpoint (const common::Loc &a_loc,
+                                 const UString &a_condition,
+                                 gint a_ignore_count,
+                                 const BreakpointSlot &a_slot,
+                                 const UString &a_cookie = "") = 0;
+
     virtual void set_breakpoint (const UString &a_path,
                                  gint a_line_num,
-                                 const UString &a_condition="",
-                                 const UString &a_cookie="") = 0;
+                                 const UString &a_condition= "",
+                                 gint a_ignore_count = 0,
+                                 const UString &a_cookie = "") = 0;
+
     virtual void set_breakpoint (const UString &a_func_name,
-                                 const UString &a_condition="",
-                                 const UString &a_cookie="") = 0;
+                                 const UString &a_condition = "",
+                                 gint a_ignore_count = 0,
+                                 const UString &a_cookie = "") = 0;
+
+    virtual void set_breakpoint (const Address &a_address,
+                                 const UString &a_condition = "",
+                                 gint a_ignore_count = 0,
+                                 const UString &a_cookie = "") = 0;
+
+    virtual void enable_breakpoint (gint a_break_num,
+                                    const BreakpointsSlot &a_slot,
+                                    const UString &a_cookie="") = 0;
 
     virtual void enable_breakpoint (gint a_break_num,
                                     const UString &a_cookie="") = 0;
@@ -1044,6 +1174,18 @@ public:
                                          gint a_ignore_count,
                                          const UString &a_cookie = "") = 0;
 
+    virtual void set_breakpoint_condition (gint a_break_num,
+                                           const UString &a_condition,
+                                           const UString &a_cookie = "") = 0;
+
+    virtual void enable_countpoint (gint a_break_num,
+                                    bool a_flag,
+                                    const UString &a_cookie ="") = 0;
+
+    virtual bool is_countpoint (gint a_break_num) const = 0;
+
+    virtual bool is_countpoint (const Breakpoint &a_breakpoint) const = 0;
+
     virtual void delete_breakpoint (const UString &a_path,
                                     gint a_line_num,
                                     const UString &a_cookie="") = 0;
@@ -1058,7 +1200,11 @@ public:
 
     virtual void list_breakpoints (const UString &a_cookie="") = 0;
 
-    virtual const map<int, BreakPoint>& get_cached_breakpoints () = 0;
+    virtual const map<int, Breakpoint>& get_cached_breakpoints () = 0;
+
+    virtual bool get_breakpoint_from_cache (int a_num,
+                                            IDebugger::Breakpoint &a_bp)
+        const = 0;
 
     virtual void choose_function_overload (int a_overload_number,
                                            const UString &a_cookie="") = 0;
@@ -1083,10 +1229,20 @@ public:
                               int a_high_frame=-1,
                               const UString &a_cookie="") = 0;
 
+    virtual void list_frames (int a_low_frame,
+                              int a_high_frame,
+                              const FrameVectorSlot &a_slot,
+                              const UString &a_cookie) = 0;
+
     virtual void list_frames_arguments (int a_low_frame=-1,
                                         int a_high_frame=-1,
                                         const UString &a_cookie="") = 0;
 
+    virtual void list_frames_arguments (int a_low_frame,
+                                        int a_high_frame,
+                                        const FrameArgsSlot &a_slot,
+                                        const UString &a_cookie) = 0;
+
     virtual void list_local_variables (const UString &a_cookie="") = 0;
 
     virtual void list_global_variables (const UString &a_cookie="") = 0;
@@ -1132,6 +1288,38 @@ public:
             const std::vector<uint8_t>& a_bytes,
             const UString& a_cookie="") = 0;
 
+    typedef sigc::slot<void,
+                       const DisassembleInfo&,
+                       const std::list<Asm>& > DisassSlot;
+
+    virtual void disassemble (size_t a_start_addr,
+                              bool a_start_addr_relative_to_pc,
+                              size_t a_end_addr,
+                              bool a_end_addr_relative_to_pc,
+                              bool a_pure_asm = true,
+                              const UString &a_cookie = "") = 0;
+
+    virtual void disassemble (size_t a_start_addr,
+                              bool a_start_addr_relative_to_pc,
+                              size_t a_end_addr,
+                              bool a_end_addr_relative_to_pc,
+                              const DisassSlot &a_slot,
+                              bool a_pure_asm = true,
+                              const UString &a_cookie = "") = 0;
+
+    virtual void disassemble_lines (const UString &a_file_name,
+                                    int a_line_num,
+                                    int a_nb_disassembled_lines,
+                                    bool a_pure_asm = true,
+                                    const UString &a_cookie = "") = 0;
+
+    virtual void disassemble_lines (const UString &a_file_name,
+                                    int a_line_num,
+                                    int a_nb_disassembled_lines,
+                                    const DisassSlot &a_slot,
+                                    bool a_pure_asm = true,
+                                    const UString &a_cookie = "") = 0;
+
     typedef sigc::slot<void, const VariableSafePtr> ConstVariableSlot;
     typedef sigc::slot<void, const VariableList> ConstVariableListSlot;
     typedef sigc::slot<void, const UString&> ConstUStringSlot;
