@@ -32,6 +32,7 @@
 #include "nmv-ui-utils.h"
 #include "nmv-i-workbench.h"
 #include "nmv-i-perspective.h"
+#include "nmv-conf-keys.h"
 
 namespace nemiver {
 
@@ -58,21 +59,30 @@ typedef SafePtr<Glib::Object, GObjectMMRef, GObjectMMUnref> GObjectMMSafePtr;
 
 struct CallStackCols : public Gtk::TreeModelColumnRecord {
     Gtk::TreeModelColumn<Glib::ustring> location;
+    Gtk::TreeModelColumn<Glib::ustring> address;
+    Gtk::TreeModelColumn<Glib::ustring> binary;
     Gtk::TreeModelColumn<Glib::ustring> function_name;
     Gtk::TreeModelColumn<Glib::ustring> function_args;
     Gtk::TreeModelColumn<Glib::ustring> frame_index_caption;
     Gtk::TreeModelColumn<int> frame_index;
     Gtk::TreeModelColumn<bool> is_expansion_row;
 
+    /// Keep the enum in an order compatible with the how the diffent
+    /// columns are appended in CallStack::Priv::build_widget.
     enum Index {
-        LOCATION=0,
-        FUNCTION_NAME,
-        FUNCTION_ARGS
+        FRAME_INDEX = 0,
+        FUNCTION_NAME_INDEX,
+        FUNCTION_ARGUMENTS_INDEX,
+        LOCATION_INDEX,
+        ADDRESS_INDEX,
+        BINARY_INDEX
     };
 
     CallStackCols ()
     {
         add (location);
+        add (address);
+	add (binary);
         add (function_name);
         add (function_args);
         add (frame_index_caption);
@@ -88,8 +98,6 @@ columns ()
     return s_cols;
 }
 
-static const char* CONF_KEY_NEMIVER_CALLSTACK_EXPANSION_CHUNK =
-    "/apps/nemiver/dbgperspective/callstack-expansion-chunk";
 static const char* COOKIE_CALL_STACK_IN_FRAME_PAGING_TRANS =
     "cookie-call-stack-in-frame-paging-trans";
 
@@ -98,6 +106,7 @@ typedef map<int, list<IDebugger::VariableSafePtr> > FrameArgsMap;
 typedef map<int, IDebugger::Frame> LevelFrameMap;
 struct CallStack::Priv {
     IDebuggerSafePtr debugger;
+    IConfMgrSafePtr conf_mgr;
     IWorkbench& workbench;
     IPerspective& perspective;
     FrameArray frames;
@@ -122,6 +131,7 @@ struct CallStack::Priv {
           IWorkbench& a_workbench,
           IPerspective& a_perspective) :
         debugger (a_dbg),
+        conf_mgr (0),
         workbench (a_workbench),
         perspective (a_perspective),
         callstack_menu (0),
@@ -140,7 +150,7 @@ struct CallStack::Priv {
 
     void init_conf ()
     {
-        IConfMgrSafePtr conf_mgr = workbench.get_configuration_manager ();
+        conf_mgr = workbench.get_configuration_manager ();
         if (!conf_mgr)
             return;
 
@@ -150,8 +160,6 @@ struct CallStack::Priv {
         if (chunk) {
             nb_frames_expansion_chunk = chunk;
         }
-        conf_mgr->add_key_to_notify
-                            (CONF_KEY_NEMIVER_CALLSTACK_EXPANSION_CHUNK);
         conf_mgr->value_changed_signal ().connect
             (sigc::mem_fun (*this, &Priv::on_config_value_changed_signal));
     }
@@ -225,7 +233,9 @@ struct CallStack::Priv {
             frame_low = frame_high + 1;
             frame_high += nb_frames_expansion_chunk;
             debugger->list_frames (frame_low, frame_high,
-                                   COOKIE_CALL_STACK_IN_FRAME_PAGING_TRANS);
+                                   sigc::mem_fun
+                                   (*this, &Priv::on_frames_listed_during_paging),
+                                   "");
             return;
         }
 
@@ -240,10 +250,28 @@ struct CallStack::Priv {
         debugger->select_frame (cur_frame_index);
     }
 
-    void finish_handling_debugger_stopped_event ()
+    void finish_update_handling ()
     {
         THROW_IF_FAIL (debugger);
-        debugger->list_frames (frame_low, frame_high);
+        debugger->list_frames (frame_low, frame_high,
+			       sigc::mem_fun (*this, &Priv::on_frames_listed),
+			       "");
+    }
+
+    void handle_update (const UString &a_cookie)
+    {
+        if (a_cookie != COOKIE_CALL_STACK_IN_FRAME_PAGING_TRANS) {
+            // Restore the frame window, in case the user changed it by
+            // requesting more call stack frames.
+            frame_low = 0;
+            frame_high = nb_frames_expansion_chunk;
+        }
+
+        if (should_process_now ()) {
+            finish_update_handling ();
+        } else {
+            is_up2date = false;
+        }
     }
 
     void on_debugger_stopped_signal (IDebugger::StopReason a_reason,
@@ -251,7 +279,7 @@ struct CallStack::Priv {
                                      const IDebugger::Frame &/*a_frame*/,
                                      int /*a_thread_id*/,
                                      int /*a_bp_num*/,
-                                     const UString &/*a_cookie*/)
+                                     const UString &a_cookie)
     {
         LOG_FUNCTION_SCOPE_NORMAL_DD;
 
@@ -264,54 +292,77 @@ struct CallStack::Priv {
             return;
         }
 
-        if (should_process_now ()) {
-            finish_handling_debugger_stopped_event ();
-        } else {
-            is_up2date = false;
-        }
+        handle_update (a_cookie);
 
         NEMIVER_CATCH
     }
 
-    void on_frames_listed_signal (const vector<IDebugger::Frame> &a_stack,
-                                  const UString &a_cookie)
+    void on_thread_selected_signal (int /*a_thread_id*/,
+                                    const IDebugger::Frame* const /*a_frame*/,
+                                    const UString& a_cookie)
+    {
+        LOG_FUNCTION_SCOPE_NORMAL_DD;
+        handle_update (a_cookie);
+    }
+
+    void
+    on_frames_listed_during_paging (const vector<IDebugger::Frame> &a_stack)
     {
         LOG_FUNCTION_SCOPE_NORMAL_DD;
 
-        NEMIVER_TRY
+        NEMIVER_TRY;
+
+        // We are in a mode where the user asked to see more stack
+        // frames.  This is named frame paging. In this case, just
+        // append the frames to the current ones. Again, the frames
+        // will be appended without arguments. We'll request the
+        // arguments right after this.
+        FrameArgsMap frames_args;
+        append_frames_to_tree_view (a_stack, frames_args);
+
+        // Okay so now, ask for frame arguments matching the frames we
+        // just received.
+        debugger->list_frames_arguments (a_stack[0].level (),
+                                         a_stack[a_stack.size () - 1].level (),
+                                         sigc::mem_fun
+                                         (*this, &Priv::on_frames_args_listed),
+                                         "");
+
+        NEMIVER_CATCH;
+    }
+
+    void
+    on_frames_listed (const vector<IDebugger::Frame> &a_stack)
+    {
+        LOG_FUNCTION_SCOPE_NORMAL_DD;
+
+        NEMIVER_TRY;
 
         THROW_IF_FAIL (debugger);
         waiting_for_stack_args = true;
 
         FrameArgsMap frames_args;
-        if (a_cookie.raw () != COOKIE_CALL_STACK_IN_FRAME_PAGING_TRANS) {
-            // Set the frame list without frame arguments,
-            // then, request IDebugger for frame arguments.
-            // When the arguments arrive, we will update the call stack
-            // with those.
-            set_frame_list (a_stack, frames_args);
-        } else {
-            // We are a mode where the user asked to see more stack frames.
-            // This is named frame paging. In this case, just append the
-            // frames to the current ones. Again, the frames will be
-            // appended without arguments. We'll request the arguments
-            // right after this.
-            append_frames_to_tree_view (a_stack, frames_args);
-        }
-        // Okay so now, ask for frame arguments matching the frames we just
-        // received.
-        debugger->list_frames_arguments (a_stack[0].level (),
-                                         a_stack[a_stack.size () - 1].level ());
+        // Set the frame list without frame arguments, then, request
+        // IDebugger for frame arguments.  When the arguments arrive,
+        // we will update the call stack with those.
+        set_frame_list (a_stack, frames_args);
 
-        NEMIVER_CATCH
+        // Okay so now, ask for frame arguments matching the frames we
+        // just received.
+        debugger->list_frames_arguments (a_stack[0].level (),
+                                         a_stack[a_stack.size () - 1].level (),
+                                         sigc::mem_fun
+                                         (*this, &Priv::on_frames_args_listed),
+                                         "");
+
+        NEMIVER_CATCH;
     }
 
-    void on_frames_params_listed_signal
-            (const map<int, list<IDebugger::VariableSafePtr> > &a_frames_args,
-             const UString &a_cookie)
+    void
+    on_frames_args_listed
+    (const map<int, IDebugger::VariableList> &a_frames_args)
     {
         LOG_DD ("frames params listed");
-        if (a_cookie.empty ()) {}
 
         if (waiting_for_stack_args) {
             update_frames_arguments (a_frames_args);
@@ -380,23 +431,33 @@ struct CallStack::Priv {
 
         NEMIVER_TRY
         if (!is_up2date) {
-            finish_handling_debugger_stopped_event ();
+            finish_update_handling ();
             is_up2date = true;
         }
         NEMIVER_CATCH
     }
 
     void on_config_value_changed_signal (const UString &a_key,
-                                         IConfMgr::Value &a_val)
+                                         const UString &a_namespace)
     {
         LOG_FUNCTION_SCOPE_NORMAL_DD;
 
+        if (!conf_mgr) {
+            return;
+        }
+
         LOG_DD ("key " << a_key << " changed");
         if (a_key == CONF_KEY_NEMIVER_CALLSTACK_EXPANSION_CHUNK) {
-            nb_frames_expansion_chunk = boost::get<int> (a_val);
+            int chunk = 0;
+            conf_mgr->get_key_value (a_key, chunk, a_namespace);
+            if (chunk) {
+                nb_frames_expansion_chunk = chunk;
+            }
         }
     }
 
+    /// Connect callback slots to the relevant signals of our
+    /// implementation of the IDebugger interface.
     void connect_debugger_signals ()
     {
         LOG_FUNCTION_SCOPE_NORMAL_DD;
@@ -405,10 +466,8 @@ struct CallStack::Priv {
 
         debugger->stopped_signal ().connect (sigc::mem_fun
                     (*this, &CallStack::Priv::on_debugger_stopped_signal));
-        debugger->frames_listed_signal ().connect (sigc::mem_fun
-                    (*this, &CallStack::Priv::on_frames_listed_signal));
-        debugger->frames_arguments_listed_signal ().connect (sigc::mem_fun
-                    (*this, &CallStack::Priv::on_frames_params_listed_signal));
+        debugger->thread_selected_signal ().connect (sigc::mem_fun
+                     (*this, &CallStack::Priv::on_thread_selected_signal));
         debugger->command_done_signal ().connect (sigc::mem_fun
                     (*this, &CallStack::Priv::on_command_done_signal));
     }
@@ -504,43 +563,42 @@ struct CallStack::Priv {
         Gtk::TreeView *tree_view = new Gtk::TreeView (store);
         THROW_IF_FAIL (tree_view);
         widget.reset (tree_view);
+	int nb_cols = 0;
         tree_view->append_column (_("Frame"), columns ().frame_index_caption);
         tree_view->append_column (_("Function"), columns ().function_name);
         tree_view->append_column (_("Arguments"), columns ().function_args);
         tree_view->append_column (_("Location"), columns ().location);
+        tree_view->append_column (_("Address"), columns ().address);
+	nb_cols = tree_view->append_column (_("Binary"), columns ().binary);
+        Gtk::TreeViewColumn *col = 0;
+	for (int i = 0; i < nb_cols; ++i) {
+            col = tree_view->get_column (i);
+            col->set_resizable ();
+            col->set_clickable ();
+	}
+
+        THROW_IF_FAIL (col = tree_view->get_column (CallStackCols::BINARY_INDEX));
+        col->get_first_cell_renderer ()->set_sensitive ();
+
         tree_view->set_headers_visible (true);
+	tree_view->columns_autosize ();
         tree_view->get_selection ()->set_mode (Gtk::SELECTION_SINGLE);
 
         on_selection_changed_connection =
             tree_view->get_selection ()->signal_changed ().connect
             (sigc::mem_fun (*this,
                             &CallStack::Priv::on_selection_changed_signal));
+
         tree_view->signal_row_activated ().connect
             (sigc::hide (sigc::hide
-             (sigc::mem_fun (*this,
-                             &CallStack::Priv::on_row_activated_signal))));
+                         (sigc::mem_fun (*this,
+                                         &CallStack::Priv::on_row_activated_signal))));
 
         tree_view->signal_expose_event ().connect_notify
             (sigc::mem_fun (this, &Priv::on_expose_event_signal));
 
         tree_view->add_events (Gdk::EXPOSURE_MASK);
 
-        Gtk::TreeViewColumn* column =
-                    tree_view->get_column (CallStackCols::FUNCTION_NAME);
-        THROW_IF_FAIL (column);
-        column->set_clickable (false);
-        column->set_reorderable (false);
-
-        THROW_IF_FAIL (column = tree_view->get_column
-                                       (CallStackCols::LOCATION));
-        column->set_clickable (false);
-        column->set_reorderable (false);
-
-        THROW_IF_FAIL (column = tree_view->get_column
-                                            (CallStackCols::FUNCTION_ARGS));
-        column->set_clickable (false);
-        column->set_reorderable (false);
-
         tree_view->signal_button_press_event ().connect_notify
             (sigc::mem_fun (*this,
                             &Priv::on_call_stack_button_press_signal));
@@ -617,7 +675,7 @@ struct CallStack::Priv {
 
         // Erase the expansion row, if it exists.
         if (store && !store->children ().empty ()) {
-            LOG_DD ("does expansion row exists ?");
+            LOG_DD ("does expansion row exist ?");
             Gtk::TreeRow last_row =
                 store->children ()[store->children ().size () - 1];
             if (last_row[columns ().is_expansion_row]) {
@@ -636,16 +694,15 @@ struct CallStack::Priv {
             store_iter = store->append ();
             (*store_iter)[columns ().is_expansion_row] = false;
             (*store_iter)[columns ().function_name] =
-                                                a_frames[i].function_name ();
+                a_frames[i].function_name ();
             if (!a_frames[i].file_name ().empty ()) {
                 (*store_iter)[columns ().location] =
                     a_frames[i].file_name () + ":"
                     + UString::from_int (a_frames[i].line ());
-            } else {
-                (*store_iter)[columns ().location] =
-                    a_frames[i].address ();
             }
-
+            (*store_iter)[columns ().address] =
+                a_frames[i].address ().to_string ();
+	    (*store_iter)[columns ().binary] = a_frames[i].library ();
             (*store_iter)[columns ().frame_index] = a_frames[i].level ();
             (*store_iter)[columns ().frame_index_caption] =
                 UString::from_int (a_frames[i].level ());
@@ -755,7 +812,9 @@ struct CallStack::Priv {
     void update_call_stack ()
     {
         THROW_IF_FAIL (debugger);
-        debugger->list_frames (0, frame_high);
+        debugger->list_frames (0, frame_high,
+                               sigc::mem_fun (*this, &Priv::on_frames_listed),
+                               "");
     }
 };//end struct CallStack::Priv
 
