@@ -27,13 +27,11 @@
 
 #include <gtkmm/widget.h>
 #include <pangomm/fontdescription.h>
-#include <libglademm.h>
 #include "common/nmv-object.h"
 #include "common/nmv-safe-ptr-utils.h"
 #include "nmv-i-debugger.h"
 
 using nemiver::common::SafePtr;
-using nemiver::common::UString;
 
 namespace nemiver {
 
