@@ -53,7 +53,7 @@ class VarInspectorDialog::Priv {
     Gtk::Button *inspect_button;
     SafePtr<VarInspector> var_inspector;
     Gtk::Dialog &dialog;
-    Glib::RefPtr<Gnome::Glade::Xml> glade;
+    Glib::RefPtr<Gtk::Builder> gtkbuilder;
     IDebuggerSafePtr debugger;
     IPerspective &perspective;
 
@@ -61,13 +61,13 @@ class VarInspectorDialog::Priv {
 public:
 
     Priv (Gtk::Dialog &a_dialog,
-          const Glib::RefPtr<Gnome::Glade::Xml> &a_glade,
+          const Glib::RefPtr<Gtk::Builder> &a_gtkbuilder,
           IDebuggerSafePtr a_debugger,
           IPerspective &a_perspective) :
         var_name_entry (0),
         inspect_button (0),
         dialog (a_dialog),
-        glade (a_glade),
+        gtkbuilder (a_gtkbuilder),
         debugger (a_debugger),
         perspective (a_perspective)
     {
@@ -81,20 +81,20 @@ public:
         LOG_FUNCTION_SCOPE_NORMAL_DD;
 
         var_name_entry =
-            ui_utils::get_widget_from_glade<Gtk::ComboBoxEntry>
-                (glade, "variablenameentry");
+            ui_utils::get_widget_from_gtkbuilder<Gtk::ComboBoxEntry>
+                (gtkbuilder, "variablenameentry");
         m_variable_history =
             Gtk::ListStore::create (get_cols ());
         var_name_entry->set_model (m_variable_history);
         var_name_entry->set_text_column (get_cols ().varname);
 
         inspect_button =
-            ui_utils::get_widget_from_glade<Gtk::Button> (glade,
+            ui_utils::get_widget_from_gtkbuilder<Gtk::Button> (gtkbuilder,
                                                           "inspectbutton");
         inspect_button->set_sensitive (false);
 
         Gtk::Box *box =
-            ui_utils::get_widget_from_glade<Gtk::Box> (glade,
+            ui_utils::get_widget_from_gtkbuilder<Gtk::Box> (gtkbuilder,
                                                        "inspectorwidgetbox");
         var_inspector.reset (new VarInspector (debugger, perspective));
         var_inspector->enable_contextual_menu (true);
@@ -144,7 +144,18 @@ public:
                         false /*don't allow duplicates*/);
     }
 
-    bool exists_in_history (const UString &a_expr)
+    /// Tests wheter if the variable expression exists in history.
+    ///
+    /// \param a_expr the expression to look for.
+    ///
+    /// \param a_iter if the return returned true and if this pointer
+    /// is non-null, then *a_iter is filled with an iterator pointing
+    /// at the expression found in history.
+    /// 
+    /// \return true if the variable expression a_expr exits in
+    /// memory, false otherwise.
+    bool exists_in_history (const UString &a_expr,
+                            Gtk::TreeModel::iterator *a_iter = 0) const
     {
         THROW_IF_FAIL (m_variable_history);
 
@@ -153,22 +164,52 @@ public:
              it != m_variable_history->children ().end ();
              ++it) {
             if ((*it)[get_cols ().varname] == a_expr) {
+                if (a_iter != 0)
+                    *a_iter = it;
                 return true;
             }
         }
         return false;
     }
 
-    void add_to_history (const UString &a_expr,
-                         bool a_prepend = true,
-                         bool a_allow_dups = true)
+    /// Erases an expression from expression history.
+    ///
+    /// \param a_iter the iterator pointing to the expression to erase
+    /// from history.
+    void erase_expression_from_history (Gtk::TreeModel::iterator &a_iter)
     {
-        if (a_expr.empty () // don't append empty exprs.
-            // don't append an expr if it exists already.
-            || (!a_allow_dups && exists_in_history (a_expr)))
-            return;
-        Gtk::TreeModel::iterator it;
+        THROW_IF_FAIL (m_variable_history);
+        m_variable_history->erase (a_iter);
+    }
 
+    /// Add an expression to variable expression history.  If the
+    /// expression already exists in history, it can either be
+    /// duplicated or be not.  Also, the new expression can be either
+    /// appended or prepended to history.
+    ///
+    /// \param a_expr the expression to add to history.
+    ///
+    /// \param a_prepend if true, prepend the expression to history.
+    ///
+    /// \param allow_dups if true, allow expressions to be present in
+    /// more than copy in history.
+    void add_to_history (const UString &a_expr,
+                         bool a_prepend = false,
+                         bool a_allow_dups = false)
+    {
+        // Don't append empty expressions.
+        if (a_expr.empty ())
+            return;
+
+        // If the expression already exists in history, remove it, so
+        // that it can be added again, as to be the latest added item
+        // to historry.
+        Gtk::TreeModel::iterator it;
+        if (!a_allow_dups
+            && exists_in_history (a_expr, &it))
+            erase_expression_from_history (it);
+
+        THROW_IF_FAIL (m_variable_history);
         if (a_prepend)
             it = m_variable_history->insert
                 (m_variable_history->children ().begin ());
@@ -188,6 +229,18 @@ public:
         }
     }
 
+    /// Set the current history of variable expression to a new one.
+    ///
+    /// \param a_hist the new history to set.
+    void set_history (const std::list<UString> &a_hist)
+    {
+        m_variable_history->clear ();
+        std::list<UString>::const_iterator it;
+        for (it = a_hist.begin (); it != a_hist.end (); ++it)
+            add_to_history (*it, /*a_prepend=*/false,
+                            /*a_allow_dups=*/false);
+    }
+
     //************************
     //<signal handlers>
     //*************************
@@ -227,13 +280,13 @@ VarInspectorDialog::VarInspectorDialog (const UString &a_root_path,
                                         IDebuggerSafePtr &a_debugger,
                                         IPerspective &a_perspective) :
     Dialog (a_root_path,
-            "varinspectordialog.glade",
+            "varinspectordialog.ui",
             "varinspectordialog")
 {
     LOG_FUNCTION_SCOPE_NORMAL_DD;
     m_priv.reset
         (new VarInspectorDialog::Priv (widget (),
-                                       glade (), a_debugger,
+                                       gtkbuilder (), a_debugger,
                                        a_perspective));
     THROW_IF_FAIL (m_priv);
 }
@@ -271,15 +324,15 @@ VarInspectorDialog::variable () const
     return m_priv->var_inspector->get_variable ();
 }
 
+/// Set the history of variable expression to a new one.
+///
+/// \param a_hist the new history.
 void
 VarInspectorDialog::set_history (const std::list<UString> &a_hist)
 {
     THROW_IF_FAIL (m_priv);
 
-    list<UString>::const_iterator it;
-    for (it = a_hist.begin (); it != a_hist.end (); ++it) {
-        m_priv->add_to_history (*it, false /*append*/);
-    }
+    m_priv->set_history (a_hist);
 }
 
 void
