@@ -22,12 +22,13 @@
  *
  *See COPYRIGHT file copyright information.
  */
-#ifndef __NMV_VAR_INSPECTOR_H__
-#define __NMV_VAR_INSPECTOR_H__
+#ifndef __NMV_VAR_INSPECTOR2_H__
+#define __NMV_VAR_INSPECTOR2_H__
 
 #include "common/nmv-object.h"
 #include "common/nmv-safe-ptr-utils.h"
 #include "nmv-i-debugger.h"
+#include "nmv-i-perspective.h"
 
 namespace Gtk {
     class Widget;
@@ -47,17 +48,21 @@ class VarInspector : public nemiver::common::Object {
     SafePtr<Priv> m_priv;
 
 public:
-    VarInspector (IDebuggerSafePtr &a_debugger);
+    VarInspector (IDebuggerSafePtr a_debugger,
+                   IPerspective &a_perspective);
     virtual ~VarInspector ();
     Gtk::Widget& widget () const;
-    void set_variable (IDebugger::VariableSafePtr &a_variable,
+    void set_variable (IDebugger::VariableSafePtr a_variable,
                        bool a_expand = false);
     void inspect_variable (const UString &a_variable_name,
                            bool a_expand = false);
     IDebugger::VariableSafePtr get_variable () const;
+    void enable_contextual_menu (bool a_flag);
+    bool is_contextual_menu_enabled () const;
     void clear ();
 };//end class VarInspector
 
 NEMIVER_END_NAMESPACE (nemiver)
-#endif //__NMV_VAR_INSPECTOR_H__
+
+#endif //__NMV_VAR_INSPECTOR2_H__
 
