@@ -35,8 +35,10 @@ NEMIVER_BEGIN_NAMESPACE (variables_utils2)
 static void update_a_variable_real (const IDebugger::VariableSafePtr a_var,
                                     const Gtk::TreeView &a_tree_view,
                                     Gtk::TreeModel::iterator &a_row_it,
+                                    bool a_truncate_type,
                                     bool a_handle_highlight,
-                                    bool a_is_new_frame);
+                                    bool a_is_new_frame,
+                                    bool a_update_members);
 
 VariableColumns&
 get_variable_columns ()
@@ -70,9 +72,19 @@ is_type_a_pointer (const UString &a_type)
     return false;
 }
 
+/// Populate the type information of the graphical node representing
+/// a variable.
+/// \param a_var_it the iterator to the graphical node representing the
+/// variable we want to set the type for
+/// \param a_type the type string to set
+/// \param a_truncate when true, if the type exceeds a certain length,
+/// truncate it. Do not truncate the variable otherwise. In any case, in
+/// presence of a multi-line type string the type is _always_ truncated to the
+/// first line.
 void
 set_a_variable_node_type (Gtk::TreeModel::iterator &a_var_it,
-                          const UString &a_type)
+                          const UString &a_type,
+                          bool a_truncate)
 {
     THROW_IF_FAIL (a_var_it);
     a_var_it->set_value (get_variable_columns ().type,
@@ -82,10 +94,11 @@ set_a_variable_node_type (Gtk::TreeModel::iterator &a_var_it,
     if (nb_lines) {--nb_lines;}
 
     UString::size_type truncation_index = 0;
-    static const UString::size_type MAX_TYPE_STRING_LENGTH = 15;
+    static const UString::size_type MAX_TYPE_STRING_LENGTH = 50;
     if (nb_lines) {
         truncation_index = a_type.find ('\n');
-    } else if (a_type.size () > MAX_TYPE_STRING_LENGTH) {
+    } else if (a_truncate
+               && (a_type.size () > MAX_TYPE_STRING_LENGTH)) {
         truncation_index = MAX_TYPE_STRING_LENGTH;
     }
     if (truncation_index) {
@@ -102,10 +115,25 @@ set_a_variable_node_type (Gtk::TreeModel::iterator &a_var_it,
     variable->type (a_type);
 }
 
+// Graphically update the value representation of a variable.
+// This updates things like the variable name, value and color (in case we
+// have to highlight the variable because it value changed in the current
+// function).
+// \param a_var the symbolic representation of the variable
+// \param a_tree_view the widget containing the graphical representation of
+// a_var.
+// \param a_iter the iterator pointing to the graphical node representing
+// a_var.
+// \param a_handle_highlight whether to highlight the variable in case its
+// new value is different from the previous one.
+// \param a_is_new_frame if true, the variable will not be highlighted.
+// otherwise, highligthing will be considered if the new variable value is
+// different from the previous one.
 void
 update_a_variable_node (const IDebugger::VariableSafePtr a_var,
                         const Gtk::TreeView &a_tree_view,
                         Gtk::TreeModel::iterator &a_iter,
+                        bool a_truncate_type,
                         bool a_handle_highlight,
                         bool a_is_new_frame)
 {
@@ -153,7 +181,7 @@ update_a_variable_node (const IDebugger::VariableSafePtr a_var,
     }
 
     (*a_iter)[get_variable_columns ().value] = a_var->value ();
-    set_a_variable_node_type (a_iter,  a_var->type ());
+    set_a_variable_node_type (a_iter,  a_var->type (), a_truncate_type);
 }
 
 
@@ -171,7 +199,8 @@ void
 update_unfolded_variable (const IDebugger::VariableSafePtr a_var,
                           const Gtk::TreeView &a_tree_view,
                           const Glib::RefPtr<Gtk::TreeStore> &a_tree_store,
-                          Gtk::TreeModel::iterator a_var_it)
+                          Gtk::TreeModel::iterator a_var_it,
+                          bool a_truncate_type)
 {
     LOG_FUNCTION_SCOPE_NORMAL_DD;
 
@@ -185,7 +214,8 @@ update_unfolded_variable (const IDebugger::VariableSafePtr a_var,
                            a_tree_view,
                            a_tree_store,
                            a_var_it,
-                           result_var_row_it);
+                           result_var_row_it,
+                           a_truncate_type);
     }
 }
 
@@ -360,12 +390,26 @@ variables_match (const IDebugger::VariableSafePtr &a_var,
     return var->equals_by_value (*a_var);
 }
 
+// Update the graphical representation of variable a_var.
+// \param a_var is the symbolic representation of the variable.
+// \param a_tree_view the widget containing the grahical representation of
+// a_var.
+// \param a_parent_row_it an iterator to a graphical ancestor of the
+// graphical representation of a_var.
+// \param a_handle_highlight whether to handle highlighting of the variable
+// or not.
+// \param a_is_new_frame must be set to true if we just entered the frame
+// containing the a_var, false otherwise.
+// \param a_update_members whether to recursively update the graphical
+// representations of the members a_var.
 bool
 update_a_variable (const IDebugger::VariableSafePtr a_var,
                    const Gtk::TreeView &a_tree_view,
                    Gtk::TreeModel::iterator &a_parent_row_it,
+                   bool a_truncate_type,
                    bool a_handle_highlight,
-                   bool a_is_new_frame)
+                   bool a_is_new_frame,
+                   bool a_update_members)
 {
     LOG_FUNCTION_SCOPE_NORMAL_DD;
     THROW_IF_FAIL (a_parent_row_it);
@@ -382,34 +426,52 @@ update_a_variable (const IDebugger::VariableSafePtr a_var,
     }
 
     update_a_variable_real (a_var, a_tree_view,
-                            row_it, a_handle_highlight,
-                            a_is_new_frame);
+                            row_it, a_truncate_type, a_handle_highlight,
+                            a_is_new_frame, a_update_members);
     return true;
 }
 
+// Subroutine of update_a_variable. See that function comments to learn
+// more.
+// \param a_var the symbolic representation of the variable we are interested in.
+// \param a_tree_view the widget containing the graphical representation of
+// a_var.
+// \param a_row_it an iterator pointing to the graphical representation of
+// a_var
+// \param a_handle_highlight whether to highlight the node pointed to by
+// a_row_it or not.
+// \param a_is_new_frame whether if we just entered the function frame
+// containing a_var or not.
+// \param a_update_members whether the function should recursively update
+// the graphical representation of the members of a_var.
 static void
 update_a_variable_real (const IDebugger::VariableSafePtr a_var,
                         const Gtk::TreeView &a_tree_view,
                         Gtk::TreeModel::iterator &a_row_it,
+                        bool a_truncate_type,
                         bool a_handle_highlight,
-                        bool a_is_new_frame)
+                        bool a_is_new_frame,
+                        bool a_update_members)
 {
     update_a_variable_node (a_var,
                             a_tree_view,
                             a_row_it,
+                            a_truncate_type,
                             a_handle_highlight,
                             a_is_new_frame);
     Gtk::TreeModel::iterator row_it;
     list<IDebugger::VariableSafePtr>::const_iterator var_it;
     Gtk::TreeModel::Children rows = a_row_it->children ();
-    //THROW_IF_FAIL (rows.size () == a_var->members ().size ());
-    //TODO: change this to handle dereferencing
-    for (row_it = rows.begin (), var_it = a_var->members ().begin ();
-         row_it != rows.end () && var_it != a_var->members ().end ();
-         ++row_it, ++var_it) {
-        update_a_variable_real (*var_it, a_tree_view,
-                                row_it, a_handle_highlight,
-                                a_is_new_frame);
+    if (a_update_members) {
+        for (row_it = rows.begin (), var_it = a_var->members ().begin ();
+             row_it != rows.end () && var_it != a_var->members ().end ();
+             ++row_it, ++var_it) {
+            update_a_variable_real (*var_it, a_tree_view,
+                                    row_it, a_truncate_type,
+                                    a_handle_highlight,
+                                    a_is_new_frame,
+                                    true /* update members */);
+        }
     }
 }
 
@@ -426,13 +488,14 @@ bool
 append_a_variable (const IDebugger::VariableSafePtr a_var,
                    const Gtk::TreeView &a_tree_view,
                    const Glib::RefPtr<Gtk::TreeStore> &a_tree_store,
-                   Gtk::TreeModel::iterator &a_parent_row_it)
+                   Gtk::TreeModel::iterator &a_parent_row_it,
+                   bool a_truncate_type)
 {
     LOG_FUNCTION_SCOPE_NORMAL_DD;
 
     Gtk::TreeModel::iterator row_it;
     return append_a_variable (a_var, a_tree_view, a_tree_store,
-                              a_parent_row_it, row_it);
+                              a_parent_row_it, row_it, a_truncate_type);
 }
 
 /// Append a variable to a variable tree view widget.
@@ -453,7 +516,8 @@ append_a_variable (const IDebugger::VariableSafePtr a_var,
                    const Gtk::TreeView &a_tree_view,
                    const Glib::RefPtr<Gtk::TreeStore> &a_tree_store,
                    Gtk::TreeModel::iterator &a_parent_row_it,
-                   Gtk::TreeModel::iterator &a_result)
+                   Gtk::TreeModel::iterator &a_result,
+                   bool a_truncate_type)
 {
     LOG_FUNCTION_SCOPE_NORMAL_DD;
     THROW_IF_FAIL (a_tree_store);
@@ -482,19 +546,24 @@ append_a_variable (const IDebugger::VariableSafePtr a_var,
     if (!a_var) {
         return false;
     }
-    update_a_variable_node (a_var, a_tree_view, row_it, true, true);
+    update_a_variable_node (a_var, a_tree_view, row_it,
+                            a_truncate_type, true, true);
     list<IDebugger::VariableSafePtr>::const_iterator it;
     if (a_var->needs_unfolding ()) {
         // Mark *row_it as needing unfolding, and add an empty
         // child node to it
         (*row_it)[get_variable_columns ().needs_unfolding] = true;
         IDebugger::VariableSafePtr empty_var;
-        append_a_variable (empty_var, a_tree_view, a_tree_store, row_it);
+        append_a_variable (empty_var, a_tree_view,
+                           a_tree_store, row_it,
+                           a_truncate_type);
     } else {
         for (it = a_var->members ().begin ();
              it != a_var->members ().end ();
              ++it) {
-            append_a_variable (*it, a_tree_view, a_tree_store, row_it);
+            append_a_variable (*it, a_tree_view,
+                               a_tree_store, row_it,
+                               a_truncate_type);
         }
     }
     a_result = row_it;
