@@ -41,29 +41,19 @@
 #include "nmv-i-workbench.h"
 #include "nmv-i-perspective.h"
 #include "nmv-i-conf-mgr.h"
+#include "nmv-conf-keys.h"
 
 using namespace std;
 using namespace nemiver;
 using namespace nemiver::common;
 
+#ifndef PACKAGE_URL
+// This is not define with autoconf < 2.64
+#define PACKAGE_URL "http://projects.gnome.org/nemiver"
+#endif
+
 NEMIVER_BEGIN_NAMESPACE (nemiver)
 
-static const UString CONF_KEY_NEMIVER_WINDOW_WIDTH =
-                "/apps/nemiver/workbench/window-width";
-static const UString CONF_KEY_NEMIVER_WINDOW_HEIGHT =
-                "/apps/nemiver/workbench/window-height";
-static const UString CONF_KEY_NEMIVER_WINDOW_POSITION_X =
-                "/apps/nemiver/workbench/window-position-x";
-static const UString CONF_KEY_NEMIVER_WINDOW_POSITION_Y =
-                "/apps/nemiver/workbench/window-position-y";
-static const UString CONF_KEY_NEMIVER_WINDOW_MAXIMIZED =
-                "/apps/nemiver/workbench/window-maximized";
-static const UString CONF_KEY_NEMIVER_WINDOW_MINIMUM_WIDTH=
-                "/apps/nemiver/workbench/window-minimum-width";
-static const UString CONF_KEY_NEMIVER_WINDOW_MINIMUM_HEIGHT=
-                "/apps/nemiver/workbench/window-minimum-height";
-
-
 class WorkbenchStaticInit {
     WorkbenchStaticInit ()
     {
@@ -138,6 +128,7 @@ public:
     Workbench (DynamicModule *a_dynmod);
     virtual ~Workbench ();
     void do_init (Gtk::Main &a_main);
+    void do_init (IConfMgrSafePtr &);
     void shut_down ();
     Glib::RefPtr<Gtk::ActionGroup> get_default_action_group ();
     Glib::RefPtr<Gtk::ActionGroup> get_debugger_ready_action_group ();
@@ -147,6 +138,7 @@ public:
     void set_title_extension (const UString &a_str);
     Glib::RefPtr<Gtk::UIManager>& get_ui_manager () ;
     IPerspective* get_perspective (const UString &a_name);
+    void set_configuration_manager (IConfMgrSafePtr &);
     IConfMgrSafePtr get_configuration_manager () ;
     Glib::RefPtr<Glib::MainContext> get_main_context () ;
     sigc::signal<void>& shutting_down_signal ();
@@ -351,15 +343,33 @@ Workbench::~Workbench ()
     LOG_D ("delete", "destructor-domain");
 }
 
+/// Initialize the workbench by setting the configuration manager it
+/// is going to use.  This function is usually called by the helper
+/// function template load_iface_and_confmgr.
+///
+/// \param a_conf_mgr the configuration manager to set.
+void
+Workbench::do_init (IConfMgrSafePtr &a_conf_mgr)
+{
+    set_configuration_manager (a_conf_mgr);
+}
+
+/// Initialize the workbench by doing all the graphical plumbling
+/// needed to setup the perspectives held by this workbench.  Calling
+/// this function is mandatory prior to using the workbench.
+///
+/// \param a_main the Gtk main object the workbench is going to use.
 void
 Workbench::do_init (Gtk::Main &a_main)
 {
     LOG_FUNCTION_SCOPE_NORMAL_DD;
 
-    DynamicModule::Loader *loader = get_dynamic_module ().get_module_loader ();
+    DynamicModule::Loader *loader =
+        get_dynamic_module ().get_module_loader ();
     THROW_IF_FAIL (loader);
 
-    DynamicModuleManager *dynmod_manager = loader->get_dynamic_module_manager ();
+    DynamicModuleManager *dynmod_manager =
+        loader->get_dynamic_module_manager ();
     THROW_IF_FAIL (dynmod_manager);
 
     m_priv->main = &a_main;
@@ -522,28 +532,31 @@ Workbench::get_perspective (const UString &a_name)
     return NULL;
 }
 
+/// Set the configuration manager
+void
+Workbench::set_configuration_manager (IConfMgrSafePtr &a_conf_mgr)
+{
+    m_priv->conf_mgr = a_conf_mgr;
+
+    NEMIVER_TRY;
+
+    m_priv->conf_mgr->register_namespace
+        (/*default nemiver namespace*/);
+    m_priv->conf_mgr->register_namespace
+        (CONF_NAMESPACE_DESKTOP_INTERFACE);
+
+    NEMIVER_CATCH;
+}
+
 IConfMgrSafePtr
 Workbench::get_configuration_manager ()
 {
     THROW_IF_FAIL (m_priv);
     if (!m_priv->conf_mgr) {
-        DynamicModule::Loader *loader =
-            get_dynamic_module ().get_module_loader ();
-        THROW_IF_FAIL (loader);
-
-        DynamicModuleManager *dynmod_manager =
-                loader->get_dynamic_module_manager ();
-        THROW_IF_FAIL (dynmod_manager);
-
-        m_priv->conf_mgr = dynmod_manager->load_iface <IConfMgr> ("gconfmgr",
-                                                                  "IConfMgr");
-        NEMIVER_TRY
-
-        m_priv->conf_mgr->set_key_dir_to_notify ("/apps/nemiver");
-        m_priv->conf_mgr->add_key_to_notify (
-                "/desktop/gnome/interface/monospace_font_name");
-
-        NEMIVER_CATCH_NOX
+        IConfMgrSafePtr new_conf_mgr = 
+            DynamicModuleManager::load_iface_with_default_manager<IConfMgr>
+            (CONFIG_MGR_MODULE_NAME, "IConfMgr");
+        set_configuration_manager (new_conf_mgr);
     }
     THROW_IF_FAIL (m_priv->conf_mgr);
     return m_priv->conf_mgr;
