@@ -5,6 +5,7 @@
 #include "common/nmv-safe-ptr-utils.h"
 #include "common/nmv-exception.h"
 #include "nmv-i-debugger.h"
+#include "nmv-debugger-utils.h"
 
 using namespace nemiver;
 using namespace nemiver::common;
@@ -118,9 +119,8 @@ test_main (int argc, char *argv[])
 
     THROW_IF_FAIL (loop);
 
-    DynamicModuleManager module_manager;
     IDebuggerSafePtr debugger =
-            module_manager.load_iface<IDebugger> ("gdbengine", "IDebugger");
+        debugger_utils::load_debugger_iface_with_confmgr ();
 
     debugger->set_event_loop_context (loop->get_context ());
 
@@ -132,7 +132,7 @@ test_main (int argc, char *argv[])
     debugger->program_finished_signal ().connect
                                             (&on_program_finished_signal);
 
-    debugger->breakpoints_set_signal ().connect
+    debugger->breakpoints_list_signal ().connect
                                             (&on_breakpoints_set_signal);
 
     debugger->stopped_signal ().connect (sigc::bind (&on_stopped_signal,
@@ -140,7 +140,8 @@ test_main (int argc, char *argv[])
 
     std::vector<UString> args, source_search_dir;
     source_search_dir.push_back (".");
-    debugger->load_program ("fooprog", args, ".", source_search_dir);
+    debugger->load_program ("fooprog", args, ".",
+                            source_search_dir, "", false);
     debugger->set_breakpoint ("main");
     debugger->run ();
     loop->run ();
